import {
  ArgumentOutOfRangeException,
  ValueObject,
} from '@mobicoop/ddd-library';
import { ActorTime } from './actor-time.value-object';
import { Step, StepProps } from './step.value-object';

/** Note:
 * Value Objects with multiple properties can contain
 * other Value Objects inside if needed.
 * */

export interface JourneyItemProps extends StepProps {
  actorTimes: ActorTime[];
}

export class JourneyItem extends ValueObject<JourneyItemProps> {
  get duration(): number {
    return this.props.duration;
  }

  get distance(): number | undefined {
    return this.props.distance;
  }

  get lon(): number {
    return this.props.lon;
  }

  get lat(): number {
    return this.props.lat;
  }

  get actorTimes(): ActorTime[] {
    return this.props.actorTimes;
  }

  protected validate(props: JourneyItemProps): void {
    // validate step props
    new Step({
      lon: props.lon,
      lat: props.lat,
      distance: props.distance,
      duration: props.duration,
    });
    if (props.actorTimes.length == 0)
      throw new ArgumentOutOfRangeException(
        'at least one actorTime is required',
      );
  }
}
