import { ArgumentInvalidException, ValueObject } from '@mobicoop/ddd-library';
import { JourneyItem } from './journey-item.value-object';

/** Note:
 * Value Objects with multiple properties can contain
 * other Value Objects inside if needed.
 * */

export interface JourneyProps {
  firstDate: Date;
  lastDate: Date;
  journeyItems: JourneyItem[];
}

export class Journey extends ValueObject<JourneyProps> {
  get firstDate(): Date {
    return this.props.firstDate;
  }

  get lastDate(): Date {
    return this.props.lastDate;
  }

  get journeyItems(): JourneyItem[] {
    return this.props.journeyItems;
  }

  protected validate(props: JourneyProps): void {
    if (props.firstDate.getUTCDay() != props.lastDate.getUTCDay())
      throw new ArgumentInvalidException(
        'firstDate week day must be equal to lastDate week day',
      );
    if (props.firstDate > props.lastDate)
      throw new ArgumentInvalidException('firstDate must be before lastDate');
    if (props.journeyItems.length < 2)
      throw new ArgumentInvalidException(
        'at least 2 journey items are required',
      );
  }
}
