import { Injectable } from '@nestjs/common';
import { Geodesic as Geolib, GeodesicClass } from 'geographiclib-geodesic';
import { GeodesicPort } from '../core/application/ports/geodesic.port';

@Injectable()
export class Geodesic implements GeodesicPort {
  private geod: GeodesicClass;

  constructor() {
    this.geod = Geolib.WGS84;
  }

  inverse = (
    lon1: number,
    lat1: number,
    lon2: number,
    lat2: number,
  ): { azimuth: number; distance: number } => {
    const { azi2: azimuth, s12: distance } = this.geod.Inverse(
      lat1,
      lon1,
      lat2,
      lon2,
    );
    if (!azimuth || !distance)
      throw new Error(
        `Inverse not found for coordinates ${lon1} ${lat1} / ${lon2} ${lat2}`,
      );
    return { azimuth, distance };
  };

  azimuth = (
    lon1: number,
    lat1: number,
    lon2: number,
    lat2: number,
  ): number => {
    const { azi2: azimuth } = this.geod.Inverse(lat1, lon1, lat2, lon2);
    if (!azimuth)
      throw new Error(
        `Azimuth not found for coordinates ${lon1} ${lat1} / ${lon2} ${lat2}`,
      );
    return azimuth;
  };

  distance = (
    lon1: number,
    lat1: number,
    lon2: number,
    lat2: number,
  ): number => {
    const { s12: distance } = this.geod.Inverse(lat1, lon1, lat2, lon2);
    if (!distance)
      throw new Error(
        `Distance not found for coordinates ${lon1} ${lat1} / ${lon2} ${lat2}`,
      );
    return distance;
  };
}
